@@ -17,6 +17,7 @@ var delay = util.delay;
 var redirectsTo = util.redirectsTo;
 var sendsJson = util.sendsJson;
 var asPromise = util.asPromise;
+var proxy = util.proxy;
 
 var testFile = path.resolve(__dirname, "assets/input.txt");
 var testFileBuffer = fs.readFileSync(testFile);
@@ -297,7 +298,7 @@ describe("follow-redirects", function () {
       });
   });
 
-  it("emits an error whem url.resolve fails", function () {
+  it("emits an error when url.resolve fails", function () {
     app.get("/a", redirectsTo("/b"));
     var urlResolve = url.resolve;
     url.resolve = function () {
@@ -1581,6 +1582,143 @@ describe("follow-redirects", function () {
           assert.equal(body[header.toLowerCase()], undefined);
         });
     });
+
+    it("passes the redirect status code to beforeRedirect", function () {
+      app.get("/a", redirectsTo("/b"));
+      app.get("/b", redirectsTo("/c", 301));
+      app.get("/c", redirectsTo("/d", 302));
+      app.get("/d", redirectsTo("/e", 303));
+      app.get("/e", redirectsTo("/f", 307));
+      app.get("/f", redirectsTo("/g", 308));
+      app.get("/g", sendsJson({ a: "b" }));
+
+      const statusCodes = [];
+
+      return server.start(app)
+        .then(asPromise(function (resolve, reject) {
+          var options = {
+            host: "localhost",
+            port: 3600,
+            path: "/a",
+            method: "GET",
+            beforeRedirect: function (_, response) {
+              statusCodes.push(response.statusCode);
+            },
+          };
+          http.get(options, concatJson(resolve, reject)).on("error", reject);
+        }))
+        .then(function (res) {
+          assert.deepEqual(res.responseUrl, "http://localhost:3600/g");
+          assert.deepEqual(res.parsedJson, { a: "b" });
+          assert.deepEqual(statusCodes, [302, 301, 302, 303, 307, 308]);
+        });
+    });
+
+    it("passes the request method to beforeRedirect", function () {
+      app.post("/a", redirectsTo("/b", 308));
+      app.post("/b", redirectsTo("/c", 301));
+      app.get("/c", redirectsTo("/d", 301));
+      app.get("/d", sendsJson({ a: "b" }));
+
+      const requestMethods = [];
+
+      return server.start(app)
+        .then(asPromise(function (resolve, reject) {
+          var options = {
+            host: "localhost",
+            port: 3600,
+            path: "/a",
+            method: "POST",
+            beforeRedirect: function (_, __, request) {
+              requestMethods.push(request.method);
+            },
+          };
+          http.get(options, concatJson(resolve, reject)).on("error", reject);
+        }))
+        .then(function (res) {
+          assert.deepEqual(res.responseUrl, "http://localhost:3600/d");
+          assert.deepEqual(res.parsedJson, { a: "b" });
+          assert.deepEqual(requestMethods, ["POST", "POST", "GET"]);
+        });
+    });
+
+    it("passes the request headers to beforeRedirect", function () {
+      app.post("/a", redirectsTo("/b"));
+      app.get("/b", redirectsTo("/c"));
+      app.get("/c", redirectsTo("/d"));
+      app.get("/d", sendsJson({ a: "b" }));
+
+      const headerChain = [];
+
+      return server.start(app)
+        .then(asPromise(function (resolve, reject) {
+          var options = {
+            host: "localhost",
+            port: 3600,
+            path: "/a",
+            method: "POST",
+            headers: {
+              "X-Foo": "bar",
+            },
+            beforeRedirect: function (optionz, __, request) {
+              optionz.headers["X-Redirect"] = `${request.url} => ${optionz.href}`;
+
+              headerChain.push(request.headers);
+            },
+          };
+          http.get(options, concatJson(resolve, reject)).on("error", reject);
+        }))
+        .then(function (res) {
+          assert.deepEqual(res.responseUrl, "http://localhost:3600/d");
+          assert.deepEqual(res.parsedJson, { a: "b" });
+          assert.deepEqual(headerChain, [
+            {
+              "Host": "localhost:3600",
+              "X-Foo": "bar",
+            },
+            {
+              "Host": "localhost:3600",
+              "X-Foo": "bar",
+              "X-Redirect": "http://localhost:3600/a => http://localhost:3600/b",
+            },
+            {
+              "Host": "localhost:3600",
+              "X-Foo": "bar",
+              "X-Redirect": "http://localhost:3600/b => http://localhost:3600/c",
+            },
+          ]);
+        });
+    });
+
+    it("passes the request URL to beforeRedirect", function () {
+      app.get("/a", redirectsTo("/b"));
+      app.get("/b", redirectsTo("/c"));
+      app.get("/c", sendsJson({ a: "b" }));
+
+      const urlChain = [];
+
+      return server.start(app)
+        .then(asPromise(function (resolve, reject) {
+          var options = {
+            host: "localhost",
+            port: 3600,
+            path: "/a",
+            method: "GET",
+            beforeRedirect: function (_, __, request) {
+              urlChain.push(request.url);
+            },
+          };
+          http.get(options, concatJson(resolve, reject)).on("error", reject);
+        }))
+        .then(function (res) {
+          assert.deepEqual(res.responseUrl, "http://localhost:3600/c");
+          assert.deepEqual(res.parsedJson, { a: "b" });
+          assert.deepEqual(urlChain, [
+            "http://localhost:3600/a",
+            "http://localhost:3600/b",
+          ]);
+        });
+    });
   });
 
   describe("when the followRedirects option is set to false", function () {
@@ -1943,6 +2081,45 @@ describe("follow-redirects", function () {
         });
     });
   });
+
+  describe("when request is going through an HTTP proxy (without a tunnel)", function () {
+    [
+      { redirectType: "absolute", redirectUrl: "http://localhost:3600/b" },
+      { redirectType: "relative", redirectUrl: "/b" },
+    ].forEach(function (testCase) {
+      it("redirects to proper URL when Location header is " + testCase.redirectType, function () {
+        app.get("/a", redirectsTo(testCase.redirectUrl));
+        app.get("/b", sendsJson({ good: "yes" }));
+        app2.port = 3601;
+        app2.all("*", proxy("localhost:3601"));
+
+        function setProxy(opts) {
+          // assuming opts is a url.parse result
+          // Update path and Host header
+          opts.path = opts.href;
+          opts.pathname = opts.href;
+
+          // Update port and host to target proxy host
+          opts.port = 3601;
+          opts.host = opts.hostname + ":" + opts.port;
+
+          // redirected requests use proxy too
+          opts.beforeRedirect = setProxy;
+        }
+        return Promise.all([server.start(app), server.start(app2)])
+          .then(asPromise(function (resolve, reject) {
+            var opts = Object.assign({}, url.parse("http://localhost:3600/a"));
+            setProxy(opts);
+
+            http.get(opts, concatJson(resolve, reject)).on("error", reject);
+          }))
+          .then(function (res) {
+            assert.deepEqual(res.parsedJson, { good: "yes" });
+            assert.deepEqual(res.responseUrl, "http://localhost:3600/b");
+          });
+      });
+    });
+  });
 });
 
 function noop() { /* noop */ }
